#include <capdev_file.h>
#include <s_util.h>

proc_print_out print_out;
proc_print_raw print_raw;
proc_add_captype add_captype;

void *lfile_init(char *interface, char *filter)
{
	struct pcap_file_header fh;
	p_cfile_priv p=malloc(sizeof(cfile_priv_t));

	if (p == NULL) return NULL;

	/* TODO: Open libpcap file */
	if ( (p->fd=open(interface, O_RDONLY)) < 0 ) {
		free(p);
		print_out(PRINT_ERROR "File %s not found" CRLF, interface);
		return NULL;
	}

	
	if ( (read(p->fd, (char *)&fh, sizeof(fh))) < sizeof(fh) ) {
		close(p->fd);
		p->fd=-1;
		free(p);
		print_out(PRINT_ERROR "Error with pcap header in %s" CRLF, interface);
		return NULL;
	}

	if ( fh.snaplen > CAP_SNAPLEN ) {
		print_out(PRINT_WARNING "Libpcap file %s has snaplen bigger "
			"than I can allow!" CRLF, interface);
	}
	
#ifdef __PLUGIN_DEBUG__
	print_out ( "LFILE: magic=%x" CRLF, fh.magic );
	print_out ( "LFILE: major=%i" CRLF, fh.version_major);
	print_out ( "LFILE: minor=%u" CRLF, fh.version_minor);
	print_out ( "LFILE: splen=%u" CRLF, fh.snaplen);
#endif
	
	p->snaplen=fh.snaplen;
	p->link=fh.linktype;
	
	return p;	
}

int lfile_end(void *priv)
{
	p_cfile_priv p=(p_cfile_priv)priv;
	if ( p == NULL ) return 0;
	close(p->fd);
	free(p);
	return 1;
}

struct packet *lfile_poll(void *priv)
{
	struct pcap_pkthdr h;
	uint len=0;
	
	p_cfile_priv p=(p_cfile_priv)priv;
	if ( p == NULL ) return NULL;
	
	if ( (read(p->fd, (char *)&h, sizeof(h))) < sizeof(h) ) {
		/* EOF */
		return NULL;
	}

	/* Make sure we dont overrun buffer */
	if ( h.caplen > CAP_SNAPLEN ) {
		len=CAP_SNAPLEN;
		lseek(p->fd, h.caplen - len, SEEK_CUR);
		print_out(PRINT_WARNING "Fixed up packet by %i" CRLF, h.caplen - len);
	}else{
		len=h.caplen;
	}
	
	memset(&p->pkt, 0, sizeof(p->pkt));
	if ( read(p->fd, (char *)&p->pkt.data, len) < len ) {
		print_out("Bugger, short read" CRLF);
		return NULL;
	}
	
	p->pkt.caplen = len;
	p->pkt.len = h.len;
	
	p->pkt.layer[LAYER_LINK].id=p->link;
	
	/* Sleep is damn dangerous segfaults when
	* parent gets a SIGINT  */
	//sleep(1);
	
	return &p->pkt;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	
	print_out=in->print_out;
	print_raw=in->print_raw;
	add_captype=in->add_captype;
	
	out->name="capture/libpcap_file";
	out->desc="Capture packets from libpcap (tcpdump) files";
	out->ver_major=1;
	out->ver_minor=0;

	if ( add_captype("libpcap_file", lfile_init, lfile_end, lfile_poll) ) {
		return PLUGIN_ERR_OK;
	}else{
		return PLUGIN_ERR_FAIL;
	}
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
