#ifndef __PACKET_HEADER_INCLUDED__
#define __PACKET_HEADER_INCLUDED__

#include <g_globals.h>
#include <pkt/ip.h>
#include <pkt/icmp.h>
#include <pkt/igmp.h>
#include <pkt/tcp.h>
#include <pkt/udp.h>
#include <pkt/arp.h>
#include <capdev.h>

#define PORT ushort
#define CAP_SNAPLEN 4096

/* From the linux kernel */
struct ethhdr{
	unsigned char   h_dest[6];       /* destination eth addr */
	unsigned char   h_source[6];     /* source ether addr    */
	unsigned short  h_proto;         /* packet type ID field */
};

typedef enum network_layer{
	LAYER_LINK,
	LAYER_NET,
	LAYER_TRANS,
	LAYER_APP,
	LAYER_MAX
}network_layer_t;

typedef struct layer {
	uint id;
	char *name;
	union {
		void	*raw;
		p_ip	ip;
		p_icmp	icmp;
		p_igmp	igmp;
		p_tcp	tcp;
		p_udp	udp;
		struct ethhdr		*mac;
	}h;
}layer_t, *p_layer;

typedef struct packet {
	time_t		time;	/* Timestamp */
	struct tm	*ltime;	/* Localised timestamp */
	
	uint		caplen;	/* Length of present data */
	uint		len;	/* Real length of data on the wire */

	p_capdev	capdev; /* The capture device that cought it */
	
	PORT		sport, dport; /* Source and dest ports */

	layer_t         layer[LAYER_MAX]; /* Network layers */
	uchar		data[CAP_SNAPLEN]; /* Full packet data */
}packet_t, *p_packet;

#endif //__PACKET_HEADER_INCLUDED__
