#include <match_ip.h>

proc_print_out		print_out;
proc_print_raw		print_raw;
proc_matcher_add	add_matcher;

u_int16_t		fb_mask;

#define CHECK_IP(x) \
	(( x->layer[LAYER_NET].id == 0x0800 ) ? 1 : 0)

int fragbits_validate(char *token, p_kv opts)
{
	p_fragbits priv;
	char *tmp;

	if ( !(priv=malloc(sizeof(fragbits_priv_t))) ) return NULL;
	memset(priv, 0, sizeof(priv));

	for (tmp=token; *tmp != '\0'; tmp++) {
		switch(*tmp) {
			case 'd':
			case 'D':
				priv->fragbits |= IP_DF;
				break;
				
			case 'm':
			case 'M':
				priv->fragbits |= IP_MF;
				break;
				
			case 'r':
			case 'R':
				priv->fragbits |= IP_RF;
				break;
			
			case '!':
				priv->mode = FBM_NOT;
				break;
				
			case '+':
				priv->mode = FBM_ANY;
				break;
				
			case '*':
				priv->mode = FBM_ALL;
				break;

			case ' ':
			case '\t':
			case ',':
				break;
			default:
				free(priv);
				return NULL;
		}
	}
	return priv;
}

int fragbits_match(void *priv, p_packet p, int negate)
{
	p_fragbits x=(p_fragbits)priv;
	u_int16_t fb=p->layer[LAYER_NET].h.ip->frag_off & fb_mask;

	if ( !CHECK_IP(p)) return 0;

	switch ( x->mode ) {
		case FBM_NORMAL:
			if ( fb == x->fragbits ) {
				MATCH_RET_POSITIVE;
			}else{
				break;
			}
		case FBM_NOT:
			if ( fb & x->fragbits == 0) {
				MATCH_RET_POSITIVE;
			}else{
				break;
			}
		case FBM_ALL:
			if ( fb & x->fragbits == x->fragbits ){
				MATCH_RET_POSITIVE;
			}else{
				break;
			}
		case FBM_ANY:
			if ( fb & x->fragbits != 0){
				MATCH_RET_POSITIVE;
			}else{
				break;
			}

		default:
			return 0;
			
	}
	
	MATCH_RET_NEGATIVE;
}
	
int tos_match(void *priv, p_packet p, int negate)
{
	int tos;
	p_range_priv x=(p_range_priv)priv;
	
	/* Make sure it's IP */
	if (!CHECK_IP(p)) return 0;

	/* Clamp values to maximum */
	if ( x->min > 65535 ) x->min=65535;
	if ( x->max > 65535 ) x->max=65535;
	
	tos = ntohs(p->layer[LAYER_NET].h.ip->tos);
	
	if ( (tos >= x->min) && (tos <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int ipid_match(void *priv, p_packet p, int negate)
{
	int ipid;
	p_range_priv x=(p_range_priv)priv;
	
	/* Make sure it's IP */
	if (!CHECK_IP(p)) return 0;

	/* Clamp values to maximum */
	if ( x->min > 65535 ) x->min=65535;
	if ( x->max > 65535 ) x->max=65535;
	
	ipid = ntohs(p->layer[LAYER_NET].h.ip->id);
	
	if ( (ipid >= x->min) && (ipid <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}


int ttl_match(void *priv, p_packet p, int negate)
{
	int ttl;
	p_range_priv x=(p_range_priv)priv;
	
	/* Make sure it's IP */
	if (!CHECK_IP(p)) return 0;

	ttl=p->layer[LAYER_NET].h.ip->ttl;

	/* Clamp values to maximum */
	if ( x->min > 255 ) x->min=255;
	if ( x->max > 255 ) x->max=255;
	
	if ( (ttl >= x->min) && (ttl <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}
uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);

	print_out=in->print_out;
	print_raw=in->print_raw;
	add_matcher=in->add_matcher;

	out->name="match/ip";
	out->desc="Some generic IP based matchers";
	out->ver_major=1;
	out->ver_minor=0;
	
	fb_mask=htons(0xE000);
	
	add_matcher("TTL",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		ttl_match);
	
	add_matcher("TOS",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		tos_match);
	
	add_matcher("IP_ID",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		ipid_match);
	
	add_matcher("FRAGBITS",
		fragbits_validate,
		in->templates.t_cleanup_ptr,
		fragbits_match);
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
